/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>

#ifdef BRCMNVRAM
#include <nvram/typedefs.h>
#include <nvram/bcmnvram.h>
#else	// !BRCMNVRAM
#include "rtl865x/rtl_cfgmgr.h"
#include <bcmnvram.h>
#endif	// BRCMNVRAM

struct varlist_s	{
	char *name,*value;
	struct timeval gt1, gt2, st1, st2;
};
typedef struct varlist_s vl_t;


// return delta between t1 and t2. unit: ms
static long time_diff (struct timeval t1, struct timeval t2)
{
	long ret = ((t2.tv_sec * 1000000 + t2.tv_usec) - (t1.tv_sec * 1000000 + t1.tv_usec)) / 1000;
	return ret;
}


int main(int argc, char *argv[])
{
	char of_name[128];
	vl_t vl[] = {
		{"time_zone","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_netmask","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_netmask","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_gateway","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_dns1_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_dns2_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_proto","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_username","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_passwd","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_idletime","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_mtu","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_service","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_ac","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_hwaddr_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_domain","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_start","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_end","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_wins_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"upnp_enable","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"http_username","",{0,0},{0,0},{0,0},{0,0}},
		{"http_passwd","",{0,0},{0,0},{0,0},{0,0}},
		{"url_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"url_keyword_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_nat_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_route_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_relay_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_demand","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outport_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_outproto_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inport_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_inproto_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_desc_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x8","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x9","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x10","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x11","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x12","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x13","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x14","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_port_x15","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x8","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x9","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x10","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x11","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x12","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x13","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x14","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_ipaddr_x15","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x8","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x9","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x10","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x11","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x12","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x13","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x14","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_lport_x15","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x8","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x9","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x10","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x11","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x12","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x13","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x14","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_proto_x15","",{0,0},{0,0},{0,0},{0,0}},
		{"fw_log_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcip_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstport_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_proto_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"misc_http_x","",{0,0},{0,0},{0,0},{0,0}},
		{"misc_httpport_x","",{0,0},{0,0},{0,0},{0,0}},
		{"misc_ping_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dmz_ip","",{0,0},{0,0},{0,0},{0,0}},
		{"fw_sipblock","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uport_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_umaxbw_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_ipaddr_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_port_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_maxbw_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"productid","",{0,0},{0,0},{0,0},{0,0}},
		{"restore_defaults","",{0,0},{0,0},{0,0},{0,0}},
		{"router_disable","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_reason_t","",{0,0},{0,0},{0,0},{0,0}},
		{"time_zone_x","",{0,0},{0,0},{0,0},{0,0}},
		{"x_WANType","",{0,0},{0,0},{0,0},{0,0}},
		{"x_RegulatoryDomain","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_ssid","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_auth_mode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_crypto","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wpa_psk","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wep_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_key","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_key1","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_key2","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_key3","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_key4","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_auth","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wep","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wpa_gtk_rekey","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_gmode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_channel","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_phrase_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_macmode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_maclist_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radius_port","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radius_key","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radius_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_rate","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_dtim","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wme","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radio","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_rts","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_frag","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_bcn","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_closed","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_rateset","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wme_no_ack","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_mode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_lazywds","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wdslist_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_wdsapply_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_mode_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_ap_density","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_gmode_protection_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_macnum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_chan_list","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_country_code","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_hwaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"bigpond","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_dnsenable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_hostname","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_heartbeat_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_txonly_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_pppoe_mru","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_hwaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_dns","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_ifname","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_ifnames","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticnum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_dns1_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_gateway_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_lease","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_static_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_domain","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_wins","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dhcp_staticmac_x","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_server_x","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_username_x","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_passwd_x","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_hostname_x","",{0,0},{0,0},{0,0},{0,0}},
		{"ddns_wildcard_x","",{0,0},{0,0},{0,0},{0,0}},
		{"DDNSStatus","",{0,0},{0,0},{0,0},{0,0}},
		{"log_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"ntp_server","",{0,0},{0,0},{0,0},{0,0}},
		{"ntp_servers","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_proto_x","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_gateway","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_hwaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_ifnames","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_ifname","",{0,0},{0,0},{0,0},{0,0}},
		{"vlan_enable","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"autofw_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x8","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x9","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x10","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x11","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x12","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x13","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x14","",{0,0},{0,0},{0,0},{0,0}},
		{"vts_desc_x15","",{0,0},{0,0},{0,0},{0,0}},
		{"sp_battle_ips","",{0,0},{0,0},{0,0},{0,0}},
		{"fw_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"misc_lpr_x","",{0,0},{0,0},{0,0},{0,0}},
		{"fw_lw_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_date_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_time_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_time_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_icmp_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_dstport_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_dstip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_default_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_date_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_srcip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_proto_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_wl_srcport_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_icmp_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_dstip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_srcport_x","",{0,0},{0,0},{0,0},{0,0}},
		{"filter_lw_default_x","",{0,0},{0,0},{0,0},{0,0}},
		{"fw_wl_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"url_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"url_time_x","",{0,0},{0,0},{0,0},{0,0}},
		{"url_date_x","",{0,0},{0,0},{0,0},{0,0}},
		{"lan_gateway_t","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_enable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_num_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_if_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_matric_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_ipaddr_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_gateway_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_netmask_x","",{0,0},{0,0},{0,0},{0,0}},
		{"sr_rip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_staticnum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_static_matric_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_static_rip_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_staticipaddr_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_staticgateway_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_default_x","",{0,0},{0,0},{0,0},{0,0}},
		{"dr_staticnetmask_x","",{0,0},{0,0},{0,0},{0,0}},
		{"firmver","",{0,0},{0,0},{0,0},{0,0}},
		{"et0macaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"lan1_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"lan1_netmask","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_proto","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_netmask","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_gateway","",{0,0},{0,0},{0,0},{0,0}},
		{"wan_etherspeed_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_nat_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_dns","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_hwaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_hostname","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_mru","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_ac","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_mtu","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_username","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_idletime","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_service","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_passwd","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_route","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_dns2_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_route_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_dns1_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_hwaddr_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_dnsenable_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_etherspeed_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_heartbeat_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_pppoe_txonly_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_dns_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_proto_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_status_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_gateway_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_netmask_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wan0_ipaddr_t","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_ssid","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_hwaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_dtim","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_key","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_key1","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_key2","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_key3","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_key4","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_crypto","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_rateset","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_maclist","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wpa_psk","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_macmode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_rate","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_auth","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_auth_mode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_rts","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_mode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_bcn","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_radius_ipaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_closed","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_lazywds","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_channel","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_radius_key","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wme_no_ack","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_radius_port","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wpa_gtk_rekey","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wep_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_phrase_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_mode_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wdslist_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_maclist_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wdsapply_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_gmode","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_gmode_protection","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_gmode_protection_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_wdsnum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl0_macnum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"os_ram_addr","",{0,0},{0,0},{0,0},{0,0}},
		{"os_flash_addr","",{0,0},{0,0},{0,0},{0,0}},
		{"et0phyaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"et1phyaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"et1macaddr","",{0,0},{0,0},{0,0},{0,0}},
		{"dl_ram_addr","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x0","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x1","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x2","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x3","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x4","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x5","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x6","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_minbw_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_uminbw_x7","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_rulenum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"qos_urulenum_x","",{0,0},{0,0},{0,0},{0,0}},
		{"preferred_lang","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radio_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radio_date_x","",{0,0},{0,0},{0,0},{0,0}},
		{"wl_radio_time_x","",{0,0},{0,0},{0,0},{0,0}},
		{"reserved_from_preferred_lang","",{0,0},{0,0},{0,0},{0,0}},
		{"sharedkeystr","",{0,0},{0,0},{0,0},{0,0}},
		{"bs_mode","",{0,0},{0,0},{0,0},{0,0}},
		{"btn_skip","",{0,0},{0,0},{0,0},{0,0}},
		{"ntp_server0","",{0,0},{0,0},{0,0},{0,0}},
		{"asus_nvram_ver","",{0,0},{0,0},{0,0},{0,0}},
		{"x_Setting","",{0,0},{0,0},{0,0},{0,0}},
	};
	FILE *fp;
	char *p;
	int pat_type = 0;
	int i, j, ret;
	int nr_vl = sizeof (vl) / sizeof (vl[0]);
	long dgt, dst;
	char *pattern[] = {"0", "11", "222", "3333", "44444", "555555", "6666666", "77777777", "888888888", "9999999999"};

	sprintf (of_name, "%s.csv", argv[0]);
	fp = fopen (of_name, "w");
	if (fp == NULL)
		perror ("Open of_name failure\n");

	if (argc > 1)
		pat_type = 1;

#ifdef BRCMNVRAM
		fprintf (fp, "brcm nvram,get(5000),set%d(5000)\n", pat_type);
#else
		fprintf (fp, "566 nvram,get(5000),set%d(5000)\n", pat_type);
#endif

	for (i = 0; i < nr_vl; ++i)	{
		gettimeofday (&(vl[i].gt1), NULL);
		for (j = 0; j < 5000; ++j)	{
			p = nvram_get (vl[i].name);
			if (p == NULL)	{
				fprintf (stderr, "nvram_get (%s) return NULL\n", vl[i].name);
				exit (1);
			} else {
				vl[i].value = p;
			}
		}
		gettimeofday (&(vl[i].gt2), NULL);

		gettimeofday (&(vl[i].st1), NULL);
		for (j = 0; j < 5000; ++j)	{
			if (pat_type == 0)
				ret = nvram_set (vl[i].name, vl[i].value);
			else
				ret = nvram_set (vl[i].name, pattern[j%10]);
		}
		gettimeofday (&(vl[i].st2), NULL);

		dgt = time_diff (vl[i].gt1, vl[i].gt2);
		dst = time_diff (vl[i].st1, vl[i].st2);
		printf ("get/set %30s elapsed %8ld,%8ldms\n", vl[i].name, dgt, dst);
		fprintf (fp, "%s,%8ld,%8ld\n", vl[i].name, dgt, dst);
	}

	fclose (fp);
	return 0;
}
